# <@LICENSE>
# Copyright 2004 Apache Software Foundation
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# </@LICENSE>

=head1 NAME

Tokenizer::MeCab - Japanese tokenizer with MeCab

=head1 SYNOPSIS

loadplugin     Mail::SpamAssassin::Plugin::Tokenizer::MeCab

=head1 DESCRIPTION

This plugin tokenizes a Japanese string with MeCab that is 
the morphological analysis engine. 

Text::MeCab 0.12 or over is required.

=cut

package Mail::SpamAssassin::Plugin::Tokenizer::MeCab;

use Mail::SpamAssassin::Plugin::Tokenizer;
use strict;
use warnings;
use bytes;

use vars qw(@ISA);
@ISA = qw(Mail::SpamAssassin::Plugin::Tokenizer);

# Have to do this so that RPM doesn't find these as required perl modules
BEGIN { require Text::MeCab; }
our $language = 'ja';

sub new {
  my $class = shift;
  my $mailsaobject = shift;

  $class = ref($class) || $class;
  my $self = $class->SUPER::new($mailsaobject, $language);
  bless ($self, $class);

  return $self;
}

sub tokenize {
  my $self = shift;
  my $text_array = shift;

  my $mecab = Text::MeCab->new();
  my @tokenized_array;
  foreach my $text (@$text_array) {
    unless ($text and $text =~ /[\x80-\xFF]/) {
      push(@tokenized_array, $text);
      next;
    }

    my @buf;
    for (my $node = $mecab->parse($text); $node; $node = $node->next) {
      push(@buf, $node->surface);
    }
    my $tokenized = join(' ', @buf);
    push(@tokenized_array, $tokenized);
  }
  return \@tokenized_array;
}

1;
